@@ -1,19 +1,23 @@
 "violin"  <- function(x,data=NULL,var=NULL,grp=NULL,grp.name=NULL, xlab=NULL, ylab=NULL,main="Density plot",vertical=TRUE,
    dots=FALSE, rain=FALSE,jitter=.05,
-alpha=1,errors=FALSE,eyes=TRUE,adjust=1,restrict=TRUE,xlim=NULL,add=FALSE,col=NULL,pch=20,scale=NULL, ...) {
+	alpha=1,errors=FALSE,eyes=TRUE,adjust=1,
+	restrict=TRUE,xlim=NULL,add=FALSE,col=NULL,pch=20,scale=NULL, ...) {#calls violinBy which is now defaulg
 
 violinBy(x=x,data=data,var=var,grp=grp,grp.name=grp.name,xlab=xlab,ylab=ylab,main=main,vertical=vertical,
-      dots=dots, rain=rain, jitter=jitter,alpha=alpha,errors=errors,eyes=eyes,adjust=adjust,restrict=restrict,xlim=xlim,add=add,col=col,pch=pch,scale=scale,...)
+      dots=dots, rain=rain, jitter=jitter,
+      alpha=alpha, errors=errors,eyes=eyes,adjust=adjust,restrict=restrict,xlim=xlim,
+      add=add,col=col,pch=pch,scale=scale,...)
 }
 
 #switched from density = 50 to alpha =.5  to speed up the plotting 
 "violinBy"  <- function(x,var=NULL,grp=NULL,data=NULL,grp.name=NULL,xlab=NULL, ylab=NULL, main="Density plot",vertical=TRUE,
-    dots=FALSE,rain=FALSE, jitter=.05,alpha= 1,errors=FALSE,eyes=TRUE,adjust=1,restrict=TRUE,xlim=NULL,add=FALSE,col=NULL,pch=20,scale=NULL, ...) {
-SCALE=.3  #how wide are the plots?
+    dots=FALSE,rain=FALSE, jitter=.05,alpha= 1,errors=FALSE,eyes=TRUE,adjust=1,
+    restrict=TRUE,xlim=NULL,add=FALSE,col=NULL,pch=20,scale=NULL, ...) {
+   if(is.null(scale)) SCALE=.3  #how wide are the plots?
 
 count.valid <- function(x) {sum(!is.na(x)) }
 
-if(is.null(col)) {col <- c("blue","red","grey","purple","green","yellow")}
+if(is.null(col)) {col <- c("blue","red","grey","purple","green","yellow")}  #default colors
 
  formula <- FALSE
    if(inherits(x, "formula")) {  ps <- fparse(x)
@@ -30,7 +34,7 @@ if(is.null(col)) {col <- c("blue","red","grey","purple","green","yellow")}
   if(!formula) {ps <- list(x = grp, y = x)  #convert from x, grp   to y ~ x format 
      if(is.null(data))  {data <- x
        
-       if(!is.null(var))  {data <- data[,var]}
+       if(!is.null(var))  {data <- data[,var, drop=FALSE]}
         var <-  ps$y <- colnames(data)
         if(is.null(var))  var <- paste("V",1:NCOL(data))
     #    colnames(x) <- colnames(data) <- ps$y  <- var}
@@ -48,12 +52,12 @@ if(is.null(col)) {col <- c("blue","red","grey","purple","green","yellow")}
           x <- x[,var , drop = FALSE]}
  nvar <- nvarg <- NCOL(x)
  x <- char2numeric(x)
-     
+ 
  col <- adjustcolor(col,alpha.f =alpha)        
 #if(!is.null(grp)) {
 # if(!is.data.frame(grp) && !is.list(grp) && (length(grp) < NROW(x))) grp <- x[,grp]
 
- if(!is.null(grp)) {stats <- describeBy(data[,ps$y],group = data[,ps$x],quant=c(0,1,.5,.25,.75),mat=TRUE,skew=FALSE) } else { stats <- describe(data[,ps$y],quant=c(0,1,.5,.25,.75),skew=FALSE)}
+ if(!is.null(grp)) {stats <- describeBy(data[,ps$y],group = data[,ps$x],quant=c(0,1,.5,.25,.75),mat=TRUE,skew=FALSE) } else { stats <- describe(data[,ps$y,drop=FALSE],quant=c(0,1,.5,.25,.75),skew=FALSE)}
   meanX <- stats[,"mean"]
   minx <- stats[,"min"]
   maxx <- stats[,"max"]
@@ -122,21 +126,27 @@ if(!is.null(scale)) {width <- scale*sqrt(nX[[i]]/tot.n.obs)/max(d[[i]]$y)} else
  #        polygon(c(d[[i]]$x,  d[[i]]$x[rev]), width* c(-d[[i]]$y,  d[[i]]$y[rev])+i,col=col[i],...)}
 zero <- rep(0,length(d[[i]]$x))
 
+
 if(rain) {if(vertical) {polygon(width*c(d[[i]]$y,  zero)+i,c( d[[i]]$x,d[[i]]$x[rev]  ),col=col[i ],...)} else {
          polygon(c(d[[i]]$x,  d[[i]]$x[rev]), width* c(d[[i]]$y,  zero)+i,col=col[i],...)}} else {
          #not rain
-   if(vertical) {polygon(width*c(-d[[i]]$y,  d[[i]]$y[rev])+i,  c(d[[i]]$x,  d[[i]]$x[rev]+i),col=col[i ],...)} else {
-         polygon(c(d[[i]]$x,  d[[i]]$x[rev]), width* c(-d[[i]]$y,  d[[i]]$y[rev])+i,col=col[i],...)}}
+     if(vertical) {polygon(width*c(-d[[i]]$y,  d[[i]]$y[rev])+i,  c(d[[i]]$x,  d[[i]]$x[rev]),col=col[i ],...)} else {
+                   polygon(c(d[[i]]$x,  d[[i]]$x[rev]), width* c(-d[[i]]$y,  d[[i]]$y[rev])+i,col=col[i],...)}}
+         
+  # if(vertical) {polygon(width*c(-d[[i]]$y,  d[[i]]$y[rev])+i,  c(d[[i]]$x,  d[[i]]$x[rev]+i),col=col[i ],...)} else {
+   #      polygon(c(d[[i]]$x,  d[[i]]$x[rev]), width* c(-d[[i]]$y,  d[[i]]$y[rev])+i,col=col[i],...)}}
          
 #draw the quartiles and  medians        
 dmd <- max(which(d[[i]]$x <= medx[i]))
 d25 <- max(which(d[[i]]$x <= Q25[i]))
 d75 <- max(which(d[[i]]$x <= Q75[i]))
+
 if(vertical) {
-segments(x0=width*d[[i]]$y[dmd] +i ,y0=d[[i]]$x[dmd],x1=-(rain-1)*width*d[[i]]$y[dmd]+i,y1=d[[i]]$x[dmd],lwd=2)
-segments(x0=width*d[[i]]$y[d25] +i ,y0=d[[i]]$x[d25],x1=-(rain-1)*width*d[[i]]$y[d25]+i,y1=d[[i]]$x[d25])
-segments(x0=width*d[[i]]$y[d75] +i ,y0=d[[i]]$x[d75],x1=-(rain-1)*width*d[[i]]$y[d75]+i,y1=d[[i]]$x[d75])
+segments(x0=width*d[[i]]$y[dmd] +i ,y0=d[[i]]$x[dmd],x1=-(1-rain)*width*d[[i]]$y[dmd]+i,y1=d[[i]]$x[dmd],lwd=2)
+segments(x0=width*d[[i]]$y[d25] +i ,y0=d[[i]]$x[d25],x1=-(1-rain)*width*d[[i]]$y[d25]+i,y1=d[[i]]$x[d25])
+segments(x0=width*d[[i]]$y[d75] +i ,y0=d[[i]]$x[d75],x1=-(1-rain)*width*d[[i]]$y[d75]+i,y1=d[[i]]$x[d75])
   } else {
+ 
   segments(y0=width*d[[i]]$y[dmd] +i ,x0=d[[i]]$x[dmd],y1=(rain-1)*width*d[[i]]$y[dmd]+i,x1=d[[i]]$x[dmd],lwd=2)
 segments(y0=width*d[[i]]$y[d25] +i ,x0=d[[i]]$x[d25],y1=(rain-1)*width*d[[i]]$y[d25]+i,x1=d[[i]]$x[d25])
 segments(y0=width*d[[i]]$y[d75] +i ,x0=d[[i]]$x[d75],y1=(rain-1)*width*d[[i]]$y[d75]+i,x1=d[[i]]$x[d75])
@@ -153,6 +163,7 @@ segments(y0=width*d[[i]]$y[d75] +i ,x0=d[[i]]$x[d75],y1=(rain-1)*width*d[[i]]$y[
            message("error bars are not implemented for horizontal plots")}
  }
  
+
 if(dots) { if(!is.null(grp)) {
 if(jitter >0) {if(length(ps$x) > 1 ) {stripchart(x[,ps$y]~ grp[,ps$x[1] ] +grp[,ps$x[2]] ,vertical=vertical,method="jitter",jitter=jitter,add=TRUE,pch=pch,...)} else {
         stripchart(x[,ps$y]~ grp[,ps$x[1] ] ,vertical=vertical,method="jitter",jitter=jitter,add=TRUE,pch=pch,...) }} else {
@@ -165,22 +176,25 @@ if(jitter >0) {if(length(ps$x) > 1 ) {stripchart(x[,ps$y]~ grp[,ps$x[1] ] +grp[,
  
                    }
                    
-if(rain)  { if(!is.null(grp)) {
- 
- tempat <- 1:length(table(grp)) -.1
-if(jitter >0) {if(length(ps$x) > 1 ) {stripchart(x[,ps$y]~ grp[,ps$x[1] ] +grp[,ps$x[2]] ,vertical=vertical,method="jitter",jitter=jitter,add=TRUE,pch=pch, at=tempat,...)} else {
+if(rain)  {
+for(i in 1:nvar) {if(!is.null(grp)) {
+  
+ tempat <- 1:length(table(grp)) -.1 + (i-1)*nvar
+if(jitter >0) {if(length(ps$x) > 1 ) {stripchart(x[,ps$y[i]]~ grp[,ps$x[1] ] +grp[,ps$x[2]] ,vertical=vertical,method="jitter",jitter=jitter,add=TRUE,pch=pch, at=tempat,...)} else {
       
-        stripchart(x[,ps$y]~ grp[,ps$x[1] ] ,vertical=vertical,method="jitter",jitter=jitter,add=TRUE,pch=pch,at=tempat, ...) }} else {
-  if(length(ps$x) > 1 ) {stripchart(x[,ps$y]~ grp[,ps$x[1]]+ grp[,ps$x[,2]],vertical=vertical,add=TRUE,pch=pch,...)} else {
-       {stripchart(x[,ps$y]~ grp[,ps$x[1]],vertical=vertical,add=TRUE,pch=pch,...)} 
+        stripchart(x[,ps$y[i]]~ grp[,ps$x[1] ] ,vertical=vertical,method="jitter",jitter=jitter,add=TRUE,pch=pch,at=tempat, ...) }} else {
+  if(length(ps$x) > 1 ) {stripchart(x[,ps$y[i]]~ grp[,ps$x[1]]+ grp[,ps$x[,2]],vertical=vertical,add=TRUE,pch=pch,...)} else {
+       {stripchart(x[,ps$y[i]]~ grp[,ps$x[1]],vertical=vertical,add=TRUE,pch=pch,...)} 
        }}
-       } else {
-       if(jitter >0) {stripchart(x ,vertical=vertical,method="jitter",jitter=jitter,add=TRUE,pch=pch,...)} else {
-        stripchart(x ,vertical=vertical,method="jitter",jitter=jitter,add=TRUE,pch=pch,...) }} 
+       }  else {
+        tempat <- 1:nvar -.1 + (i-1)*nvar
+       if(jitter >0) {stripchart(x ,vertical=vertical,method="jitter",jitter=jitter,add=TRUE,pch=pch,at=tempat,...)} else {
+        stripchart(x ,vertical=vertical,method="jitter",jitter=jitter,add=TRUE,pch=pch,at=tempat,...) }} 
  
                    }
        
        }
+       }
         
 #created March 10, 2014 following a discussion of the advantage of showing distributional values
 #modified March 22, 2014 to add the grouping variable
