@@ -420,7 +420,19 @@ if(is.null(names(fa.results)) )  {temp <- fa.results   #the matrix form
      return(contents)
     }
   
-  "item.lookup" <- 
+  #created 1/15/24
+"itemSort" <- function(m, dictionary, ascending=TRUE, digits = 2) {
+ contents <- lookup(rownames(m), y=dictionary)
+ results <- data.frame(means=round(m,digits=digits),contents)
+ results <- psychTools::dfOrder(results,ascending=ascending)
+ return(results)
+}
+ 
+
+
+   
+ #patched   1/15/24 to give means  
+  "item.lookup" <-             
 function (f,m, dictionary,cut=.3, digits = 2) {
    # f <- fa.sort(f)
     none<- NULL   #A strange requirement of R 4.0
@@ -461,7 +473,9 @@ function (f,m, dictionary,cut=.3, digits = 2) {
         ord <- rownames(f)
         nfact <- ncol(f)
     }
-    means <- m[ord]   #incorrectly added a comma to allow it sort dataframes 6/22/21
+   if(!is.null(dim(m))) {
+    means <- m[ord,,drop=FALSE]   #fixed 1/15/24  
+    } else {means <- m}
     f <- data.frame(unclass(f),means=means)
 
     contents <- lookup(rownames(f), y=dictionary)
@@ -600,7 +614,7 @@ text(x,y-2,"Unreliable but Valid")
 #finally added the char2numeric so it will not choke on character variables   1/3/21
 
 #added the cor option 11/18/23
-"cor2" <- function(x,y=NULL,digits=2,use="pairwise",method="pearson",cor="cor") {
+"cor2" <- function(x,y=NULL,digits=2,use="pairwise",method="pearson",cor="cor",show=TRUE) {
 multi <- FALSE
 if(is.list(x) && is.null(y)) {multi <- TRUE
  n <- length(x)
@@ -628,7 +642,9 @@ mixed = {R <- mixedCor(cbind(x,y))$rho}
 
 )
 }
+if(show) {
 if(multi) {lowerMat(R,digits) } else {print(round(R,digits))}
+}
 invisible(R)}
 
 levels2numeric <- function(x) {
@@ -667,19 +683,19 @@ pretty}
      }
 
 "char2numeric" <- function(x,flag=TRUE) {
-
+#convert non numeric (factors, characters, logical) to numeric
  nvar <- NCOL(x)
  for(i in 1:nvar) {   
         if(!is.numeric(x[[i]] ))  {
                                   if(is.factor(unlist(x[[i]])) | is.character(unlist(x[[i]]))) {  x[[i]] <- as.numeric(as.factor(x[[i]])) 
-                          } else {x[[i]] <- NA}
+                          } else {if(is.logical(x[[i]])) {x[[i]] <- x[[i] ] +0 } else { x[[i]] <- NA}
+                          }
                          if(flag) colnames(x)[i] <- paste0(colnames(x)[i],"*")
-                          } 
-                          
+                          }                     
               } 
               invisible(x)} 
               
-              
+# added the is.logical test  23/06/24
 #added June 25, 2020 to handle the case of numeric data stored as characters  
 #added flag and colnames option  1/2/21             
 "nchar2numeric" <- function(x,flag=TRUE) {
